#ifndef IS_CTR_MEDIAFILER_LIB_H_
#define IS_CTR_MEDIAFILER_LIB_H_

namespace is { namespace mediafiler {

// Allocator
typedef void* (*Alloc)(size_t size, s32 alignment);
typedef void  (*Free)(void* p);

// Callback
struct CallbackArgs
{
	// Type
	enum Type
	{
        READ,					// Copy a file to PC from CTR
        WRITE,					// Copy a file to CTR from PC
        CREATE,					// Create a new file or folder on the CTR
        DELETE,					// Delete a file or folder on the CTR
        RENAME					// Rename a file or folder on the CTR
	};

	// Variable
	Type			m_type;			// Operation type
	bool			m_isDirectory;	// Operation target is directory?
	NN_PADDING2;
    const wchar_t*	m_fileName;		// Operation target file
    const wchar_t*	m_oldName;		// Used RENAME only. Other is NULL.
};
typedef void (*CallbackFunction)(void* userArgs, const CallbackArgs* callbackArgs);

// You must call nn::hio::CTR::Initialize
// before using this library.

// Initializes the library.
// Returns false if it could not initialize the library.
// Default value of transferBufferSize is a saving memory setting. (4 * 1024)
// If have a margin in memory and you would like to transfer the large file, 
// you will be file transfer speed up when specify a large value of double exponentiation. (256 * 1024, 1024 * 1024 etc.)
// Return false if it could not specify a double exponentiation to transferBufferSize.
bool Initialize(Alloc allocFromMainMemory, Free freeFromMainMemory, Alloc allocFromDeviceMemory, Free freeFromDeviceMemory, size_t transferBufferSize = 4 * 1024);

// Sets the expanded save data ID.
// Default value is 0xff3ff.
void SetExtSaveDataId(nn::fs::ExtSaveDataId id);

// Register a callback function when occurs of file operation from PC.
void SetCallback(CallbackFunction function, void* userArgs);

// Starts server processing.
// Returns false if it could not start server processing.
// No other function in this library allocates memory.
bool StartServer();

// Ends server processing.
void EndServer();

// Finalizes the library.
void Finalize();

}}

#endif
